@@ -1,7 +1,12 @@
+# encoding: utf-8
+
 import json
+
 import click
+
 import googleanalytics as ga
-from common import authenticated, cli
+from .common import authenticated, cli
+
 
 @cli.command()
 @click.option('-s', '--show', type=click.Choice(['pairs', 'json']))
@@ -12,8 +17,8 @@ def authorize(identity, accounts, show=False):
     if show:
         if show == 'pairs':
             for key, value in credentials.serialize().items():
-                print key, '=', value
+                print(key, '=', value)
         else:
-            print json.dumps(credentials.serialize(), indent=4)
+            print(json.dumps(credentials.serialize(), indent=4))
 
     return credentials
