@@ -1,5 +1,8 @@
+# encoding: utf-8
+
 import googleanalytics as ga
-from common import authenticated, cli
+from .common import authenticated, cli
+
 
 @cli.command()
 @authenticated
