@@ -1,10 +1,19 @@
+# encoding: utf-8
+
+"""
+"""
+
 from copy import deepcopy
 import collections
+import time
+from functools import partial
+
 import addressable
 import inspector
-import utils
-import account
-import columns
+
+from .columns import Column, Segment
+from . import utils
+from . import errors
 
 
 class Report(object):
@@ -12,18 +21,22 @@ class Report(object):
         self.raw = []
         self.queries = []
 
-        registry = query.profile.webproperty.account.columns
+        registry = query.api.all_columns
         headers = [registry[header['name']] for header in raw['columnHeaders']]
         slugs = [header.pyslug for header in headers]
         self.row_cls = collections.namedtuple('Row', slugs)
         self.headers = addressable.List(headers, 
             indices=registry.indexed_on, insensitive=True)
+        self.metrics = set()
+        self.dimensions = set()
         self.rows = []
         self.append(raw, query)
 
     def append(self, raw, query):
         self.raw.append(raw)
         self.queries.append(query)
+        self.metrics.update(query.raw['metrics'])
+        self.dimensions.update(query.raw.get('dimensions', []))
         self.is_complete = not 'nextLink' in raw
 
         casters = [column.cast for column in self.headers]
@@ -39,7 +52,7 @@ class Report(object):
         self.totals = raw['totalsForAllResults']
         # more intuitive when querying for just a single metric
         self.total = raw['totalsForAllResults'].values()[0]
-        # print self.totals
+        # print(self.totals)
 
     @property
     def first(self):
@@ -50,12 +63,20 @@ class Report(object):
         return self.rows[-1]
 
     @property
-    def values(self):
-        return self.first
+    def value(self):
+        if len(self.rows) == 1:
+            return self.values[0]
+        else:
+            raise ValueError("This report contains multiple rows or metrics. Please use `rows`, `first`, `last` or a column name.")
 
     @property
-    def value(self):
-        return self.first[0]
+    def values(self):
+        if len(self.metrics) == 1:
+            raw_metric = list(self.metrics).pop()
+            metric = self.headers[raw_metric]
+            return self[metric]
+        else:
+            raise ValueError("This report contains multiple metrics. Please use `rows`, `first`, `last` or a column name.")
 
     def serialize(self):
         serialized = []
@@ -68,6 +89,8 @@ class Report(object):
 
     def __getitem__(self, key):
         try:
+            if isinstance(key, Column):
+                key = key.slug
             i = self.headers.index(key)
             return [row[i] for row in self.rows]
         except ValueError:
@@ -79,81 +102,83 @@ class Report(object):
     def __len__(self):
         return len(self.rows)
 
-
-def condition(value):
-    return "condition::" + value
-
-def sequence(value):
-    return "sequence::" + value
-
-def all(*values):
-    return condition(";".join(values))
-
-def any(*values):
-    return condition(",".join(values))
-
-def followed_by(*values):
-    return sequence(";->>".join(values))
-
-def immediately_followed_by(*values):
-    return sequence(";->".join(values))
+    # TODO: would be cool if we could split up headers 
+    # into metrics vs. dimensions so we could say 
+    # "pageviews by day, browser"
+    # (also see `title` and `description` on query objects)
+    def __repr__(self):
+        headers = [header.name for header in self.headers]
+        return '<googleanalytics.query.Report object: {}'.format(', '.join(headers))
 
 
 class Query(object):
-    def __init__(self, profile, metrics=[], dimensions=[], meta={}, title=None):
+    """
+    Return a query for certain metrics and dimensions.
+
+    ```python
+    # pageviews (metric) as a function of geographical region
+    profile.core.query('pageviews', 'region')
+    # pageviews as a function of browser
+    profile.core.query(['pageviews'], ['browser'])
+    ```
+
+    The returned query can then be further refined using 
+    all methods available on the `CoreQuery` object, such as 
+    `limit`, `sort`, `segment` and so on.
+
+    Metrics and dimensions may be either strings (the column id or
+    the human-readable column name) or Metric or Dimension 
+    objects.
+
+    Metrics and dimensions specified as a string are not case-sensitive.
+
+    ```python
+    profile.query('PAGEVIEWS')
+    ```
+
+    If specifying only a single metric or dimension, you can 
+    but are not required to wrap it in a list.
+    """
+
+    _lock = 0
+
+    def __init__(self, api, metrics=[], dimensions=[], meta=None, title=None):
         self._title = title
-        self.raw = {'ids': 'ga:' + profile.id}
+        self.raw = {'ids': 'ga:' + api.profile.id}
         self.meta = {}
-        self.meta.update(meta)
-        self.profile = profile
-        self.webproperty = profile.webproperty
-        self.account = profile.webproperty.account
+        self.meta.update(meta or {})
+        self.api = api
+        self.profile = api.profile
+        self.webproperty = api.profile.webproperty
+        self.account = api.profile.webproperty.account
         self._report = None
         self._specify(metrics=metrics, dimensions=dimensions)
 
+    # no not execute more than one query per second
+    def _wait(self):
+        now = time.time()
+        elapsed = now - self._lock
+        wait = max(0, 1 - elapsed)
+        time.sleep(wait)
+        self._lock = time.time()
+        return wait
+
+    # REFACTOR
     def _serialize_criterion(criterion):
         pattern = r'(?P<identifier>[\w:]+)((?P<operator>[\!\=\>\<\@\~]+)(?P<value>[\w:]+))?'
         match = re.match(pattern, criterion)
         identifier = match.group('identifier')
         operator = match.group('operator') or ''
         value = match.group('value') or ''
-        column = self._serialize_column(identifier)
+        column = self.api.all_columns.serialize(identifier)
         return column + operator + value
 
-    def _normalize_column(self, value):
-        if isinstance(value, account.Column):
-            return value
-        else:
-            return self.account.columns[value]
-
-    def _serialize_column(self, value):
-        return self._normalize_column(value).id
-
-    def _serialize_columns(self, values):
-        if not isinstance(values, (list, tuple)):
-            values = [values]
-
-        return [self._serialize_column(value) for value in values]
-
-    def _normalize_segment(self, value):
-        if isinstance(value, account.Segment):
-            return value
-        else:
-            return self.account.segments[value]
-    
-    def _serialize_segment(self, value):
-        return self._normalize_segment(value).id
-
-    def _serialize(self, obj):
-        if isinstance(obj, list):
-            return [self._serialize(el) for el in obj]
-        elif isinstance(obj, account.Column):
-            return obj.id
-        else:
-            return obj
+    @property
+    def endpoint(self):
+        return self.account.service.data().ga()
 
     def clone(self):
-        query = self.__class__(profile=self.profile, meta=self.meta)
+        query = self.__class__(api=self.api, meta=self.meta)
         query.raw = deepcopy(self.raw)
         query._report = None
         return query
@@ -162,18 +187,20 @@ class Query(object):
     def set(self, key=None, value=None, **kwargs):
         """
         `set` is a way to add raw properties to the request, 
-        for features that python-google-analytics does not 
+        for features that this module does not 
         support or supports incompletely. For convenience's 
         sake, it will serialize Column objects but will 
         leave any other kind of value alone.
         """
 
+        serialize = partial(self.api.columns.serialize, greedy=False)
+
         if key and value:
-            self.raw[key] = self._serialize(value)
+            self.raw[key] = serialize(value)
         elif key or kwargs:
             properties = key or kwargs
             for key, value in properties.items():
-                self.raw[key] = self._serialize(value)
+                self.raw[key] = serialize(value)
         else:
             raise ValueError(
                 "Query#set requires a key and value, a properties dictionary or keyword arguments.")
@@ -181,8 +208,10 @@ class Query(object):
         return self
 
     def _specify(self, metrics=[], dimensions=[]):
-        metrics = self._serialize_columns(metrics)
-        dimensions = self._serialize_columns(dimensions)
+        serialize = partial(self.api.columns.serialize, wrap=True)
+
+        metrics = serialize(metrics)
+        dimensions = serialize(dimensions)
         self.raw.setdefault('metrics', []).extend(metrics)
         self.raw.setdefault('dimensions', []).extend(dimensions)
 
@@ -273,12 +302,12 @@ class Query(object):
         sorts = []
 
         for column in columns:          
-            if isinstance(column, account.Column):
+            if isinstance(column, Column):
                 ascending = False
                 identifier = column.id
             elif isinstance(column, basestring):
                 ascending = column.startswith('-')
-                identifier = self.account.columns[column.lstrip('-')].id
+                identifier = self.api.columns[column.lstrip('-')].id
             else:
                 raise ValueError()
 
@@ -298,6 +327,74 @@ class Query(object):
         object and the `all` and `any` functions. """
         self.raw['filters'] = value
 
+    def build(self):
+        raw = deepcopy(self.raw)
+        raw['metrics'] = ','.join(self.raw['metrics'])
+        
+        if len(raw['dimensions']):
+            raw['dimensions'] = ','.join(self.raw['dimensions'])
+        else:
+            raw['dimensions'] = None
+
+        return raw
+
+    def execute(self):
+        raw = self.build()
+
+        try:
+            self._wait()
+            response = self.endpoint.get(**raw).execute()
+        except Exception as err:
+            if isinstance(err, TypeError):
+                parameters = utils.paste(self.raw, '\t', '\n', pad=True)
+                message = err.message
+                diagnostics = utils.format(
+                    """
+                    {message}
+
+                    The query you submitted was:
+
+                    {parameters}
+                    """, message=message, parameters=parameters)
+                raise errors.InvalidRequestError(diagnostics)
+            else:
+                raise err
+
+        return Report(response, self)
+
+    @property
+    def report(self):
+        if not self._report:
+            self._report = self.get()
+        return self._report
+
+    # various lazy-loading shortcuts
+    @property
+    def rows(self):
+        return self.report.rows
+
+    @property
+    def first(self):
+        return self.report.first
+
+    @property
+    def last(self):
+        return self.report.last
+
+    @property
+    def value(self):
+        return self.report.value
+
+    @property
+    def values(self):
+        return self.report.values
+
+    def serialize(self):
+        return self.report.serialize()
+
+    def __repr__(self):
+        return "<googleanalytics.query.{} object: {} ({})>".format(self.__class__.__name__, self.title, self.profile.name)
+
 
 class CoreQuery(Query):
     """
@@ -323,8 +420,8 @@ class CoreQuery(Query):
 
     ```python
     base = profile.query('pageviews')
-    january = base.daily('2014-01-01', months=1).execute()
-    february = base.daily('2014-02-01', months=1).execute()
+    january = base.daily('2014-01-01', months=1).get()
+    february = base.daily('2014-02-01', months=1).get()
     ```
     """
 
@@ -394,6 +491,7 @@ class CoreQuery(Query):
 
         [query]: https://developers.google.com/analytics/devguides/reporting/core/v3/reference#q_summary
         """
+
         start, stop = utils.daterange(start, stop, months, days)
 
         self.raw.update({
@@ -477,6 +575,10 @@ class CoreQuery(Query):
         kwargs['granularity'] = 'year'
         return self.range(*vargs, **kwargs)
 
+    @inspector.implements(range)
+    def lifetime(self, *vargs, **kwargs):
+        return self.range(*vargs, **kwargs)
+
     @utils.immutable
     def step(self, maximum):
         """
@@ -566,21 +668,11 @@ class CoreQuery(Query):
         if type:
             value = "{type}::{value}".format(type=type, value=value)
         else:
-            value = self._serialize_segment(value)
+            value = self.api.segments.serialize(value)
 
         self.raw['segment'] = value
         return self
 
-    def live(self):
-        """
-        Turn a regular query into one for the live API.
-
-        **Note:** a placeholder, not implemented yet.
-        """
-        # add in metrics, dimensions, sort, filters
-        raise NotImplementedError()
-        return RealTimeQuery(metrics=self.metrics, dimensions=self.dimensions)
-
     @utils.immutable
     def next(self):
         """
@@ -592,24 +684,14 @@ class CoreQuery(Query):
         self.raw['start_index'] = start
         return self
 
-    def _execute(self):
-        raw = deepcopy(self.raw)
-        raw['metrics'] = ','.join(self.raw['metrics'])
-        raw['dimensions'] = ','.join(self.raw['dimensions'])
-
-        service = self.account.service
-        response = service.data().ga().get(**raw).execute()
-        
-        return Report(response, self)        
-
-    def execute(self):
+    def get(self):
         """
         Run the query and return a `Report`.
 
-        Execute transparently handles paginated results, so even for results that 
+        This method transparently handles paginated results, so even for results that 
         are larger than the maximum amount of rows the Google Analytics API will 
         return in a single request, or larger than the amount of rows as specified 
-        through `CoreQuery#step`,  execute will leaf through all pages,  
+        through `CoreQuery#step`, `get` will leaf through all pages,  
         concatenate the results and produce a single Report instance.
         """
 
@@ -619,7 +701,7 @@ class CoreQuery(Query):
         is_enough = False
 
         while not (is_enough or is_complete):
-            chunk = cursor._execute()
+            chunk = cursor.execute()
 
             if report:
                 report.append(chunk.raw[0], cursor)
@@ -632,49 +714,44 @@ class CoreQuery(Query):
 
         return report
 
-    @property
-    def report(self):
-        if not self._report:
-            self._report = self.execute()
-        return self._report
-
-    # various lazy-loading shortcuts
-    @property
-    def rows(self):
-        return self.report.rows
-
-    @property
-    def first(self):
-        return self.report.first
-
-    @property
-    def last(self):
-        return self.report.last
-
-    @property
-    def value(self):
-        return self.report.value
-
-    @property
-    def values(self):
-        return self.report.values
-
-    def serialize(self):
-        return self.report.serialize()
-
-    def __repr__(self):
-        return "<Query: {} ({})>".format(self.title, self.profile.name)
 
 
 class RealTimeQuery(Query):
     """
-    A query against the [Google Analytics Live API][live].
+    A query against the [Google Analytics Real Time API][realtime].
 
-    **Note:** a placeholder, not implemented yet.
+    **Note:** brand new! Please test and submit any issues to GitHub.
 
-    [live]: https://developers.google.com/analytics/devguides/reporting/realtime/v3/reference/data/realtime#resource
+    [realtime]: https://developers.google.com/analytics/devguides/reporting/realtime/v3/reference/data/realtime#resource
     """
 
+    @property
+    def endpoint(self):
+        return self.account.service.data().realtime()
+
+    @utils.immutable
+    def limit(self, maximum):
+        """
+        Return a new query, limited to a certain number of results.
+
+        Unlike core reporting queries, you cannot specify a starting 
+        point for live queries, just the maximum results returned.
+
+        ```python
+        # first 50
+        query.limit(50)
+        ```
+        """
+
+        self.meta['limit'] = maximum
+        self.raw.update({
+            'max_results': maximum, 
+        })
+        return self
+
+    def get(self):
+        return self.execute()
+
 
 def describe(profile, description):
     """
@@ -686,7 +763,9 @@ def describe(profile, description):
     Mostly useful if you'd like to put your queries
     in a file, rather than in Python code.
     """
-    return refine(profile.query(), description)
+    api_type = description.get('type', 'core')
+    api = getattr(profile, api_type)
+    return refine(api.query(), description)
 
 def refine(query, description):
     """
