@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 """
 These unit tests are somewhat limited in scope because they need 
 to work with any Google Analytics data. Therefore, we mainly test
@@ -17,4 +19,4 @@ at least one Google Analytics domain set up.
 import googleanalytics as ga
 import unittest
 import datetime
-import meta, query, experimental
\ No newline at end of file
+from . import meta, query, experimental
\ No newline at end of file
